@@ -543,6 +543,571 @@ class TestXLIFFfile(test_base.TestTranslationStore):
         assert newxfile.getfilenode("file1") is not None
         assert not newxfile.getfilenode("foo")
 
+    def test_preserve_groups_when_adding_units(self):
+        """Test that groups are preserved when adding units from one store to another."""
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="file1.txt" source-language="en" datatype="plaintext">
+        <body>
+            <group id="group1" restype="x-gettext-domain">
+                <trans-unit id="hello">
+                    <source>Hello</source>
+                </trans-unit>
+            </group>
+            <group id="group2">
+                <trans-unit id="world">
+                    <source>World</source>
+                </trans-unit>
+            </group>
+        </body>
+    </file>
+</xliff>"""
+
+        expected_output = b"""<?xml version="1.0" encoding="UTF-8"?>
+<xliff xmlns="urn:oasis:names:tc:xliff:document:1.1" version="1.1">
+  <file original="file1.txt" source-language="en" datatype="plaintext" target-language="fr">
+    <body>
+      <group id="group1" restype="x-gettext-domain">
+        <trans-unit id="hello">
+          <source>Hello</source>
+        </trans-unit>
+      </group>
+      <group id="group2">
+        <trans-unit id="world">
+          <source>World</source>
+        </trans-unit>
+      </group>
+    </body>
+  </file>
+</xliff>
+"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        assert len(source.units) == 2
+
+        # Add units to a new translation store
+        translation = xliff.xlifffile()
+        translation.sourcelanguage = "en"
+        translation.targetlanguage = "fr"
+
+        for unit in source.units:
+            translation.addunit(unit)
+
+        # Verify the structure matches expected
+        serialized = bytes(translation)
+        assert serialized == expected_output
+
+        # Also verify by parsing and checking structure
+        translation_reloaded = xliff.xlifffile.parsestring(serialized)
+        assert len(translation_reloaded.units) == 2
+        for unit in translation_reloaded.units:
+            parent = unit.xmlelement.getparent()
+            assert etree.QName(parent).localname == "group"
+
+    def test_preserve_multiple_files_and_groups(self):
+        """Test that both files and groups are preserved."""
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="file1.txt" source-language="en" datatype="plaintext">
+        <body>
+            <group id="group1">
+                <trans-unit id="hello">
+                    <source>Hello</source>
+                </trans-unit>
+            </group>
+        </body>
+    </file>
+    <file original="file2.txt" source-language="en" datatype="plaintext">
+        <body>
+            <group id="group2">
+                <trans-unit id="foo">
+                    <source>Foo</source>
+                </trans-unit>
+            </group>
+        </body>
+    </file>
+</xliff>"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        assert len(source.units) == 2
+
+        # Add units to a new translation store
+        translation = xliff.xlifffile()
+        translation.sourcelanguage = "en"
+        translation.targetlanguage = "fr"
+
+        for unit in source.units:
+            translation.addunit(unit)
+
+        # Verify both files and groups are preserved
+        assert translation.getfilenames() == ["file1.txt", "file2.txt"]
+        serialized = bytes(translation)
+        translation_reloaded = xliff.xlifffile.parsestring(serialized)
+
+        # Verify structure matches original
+        assert translation_reloaded.getfilenames() == ["file1.txt", "file2.txt"]
+        assert len(translation_reloaded.units) == 2
+
+        # Check that each unit is in the correct file and group
+        for i, unit in enumerate(translation_reloaded.units):
+            parent = unit.xmlelement.getparent()
+            assert etree.QName(parent).localname == "group"
+            file_node = None
+            for ancestor in unit.xmlelement.iterancestors():
+                if ancestor.get("original"):
+                    file_node = ancestor.get("original")
+                    break
+            if i == 0:
+                assert file_node == "file1.txt"
+                assert parent.get("id") == "group1"
+            else:
+                assert file_node == "file2.txt"
+                assert parent.get("id") == "group2"
+
+    def test_add_unit_to_existing_group(self):
+        """Test that new units are added to existing groups when appropriate."""
+        # Start with translation that has one group
+        translation_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="file1.txt" source-language="en" datatype="plaintext" target-language="fr">
+        <body>
+            <group id="group1">
+                <trans-unit id="hello">
+                    <source>Hello</source>
+                    <target>Bonjour</target>
+                </trans-unit>
+            </group>
+        </body>
+    </file>
+</xliff>"""
+
+        translation = xliff.xlifffile.parsestring(translation_xliff)
+
+        # Add a new unit from a source that has the same group
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="file1.txt" source-language="en" datatype="plaintext">
+        <body>
+            <group id="group1">
+                <trans-unit id="world">
+                    <source>World</source>
+                </trans-unit>
+            </group>
+        </body>
+    </file>
+</xliff>"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        new_unit = source.units[0]
+
+        # Add the new unit
+        translation.addunit(new_unit)
+
+        # Verify the new unit is in the same group
+        assert len(translation.units) == 2
+        serialized = bytes(translation)
+        translation_reloaded = xliff.xlifffile.parsestring(serialized)
+
+        # Both units should be in group1
+        for unit in translation_reloaded.units:
+            parent = unit.xmlelement.getparent()
+            assert etree.QName(parent).localname == "group"
+            assert parent.get("id") == "group1"
+
+    def test_add_unit_to_different_file(self):
+        """Test adding units with different file than existing ones."""
+        # Start with translation that has one file
+        translation_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="file1.txt" source-language="en" datatype="plaintext" target-language="fr">
+        <body>
+            <trans-unit id="hello">
+                <source>Hello</source>
+                <target>Bonjour</target>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+
+        translation = xliff.xlifffile.parsestring(translation_xliff)
+        assert translation.getfilenames() == ["file1.txt"]
+
+        # Add a new unit from a different file
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="file2.txt" source-language="en" datatype="plaintext">
+        <body>
+            <trans-unit id="world">
+                <source>World</source>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        new_unit = source.units[0]
+
+        # Add the new unit
+        translation.addunit(new_unit)
+
+        # Verify both files exist
+        assert len(translation.units) == 2
+        assert set(translation.getfilenames()) == {"file1.txt", "file2.txt"}
+
+        # Verify structure is correct
+        serialized = bytes(translation)
+        translation_reloaded = xliff.xlifffile.parsestring(serialized)
+        assert set(translation_reloaded.getfilenames()) == {"file1.txt", "file2.txt"}
+
+        # Verify each unit is in the correct file
+        for unit in translation_reloaded.units:
+            unit_id = unit.getid()
+            if "hello" in unit_id:
+                assert unit_id.startswith("file1.txt")
+            elif "world" in unit_id:
+                assert unit_id.startswith("file2.txt")
+
+    def test_mixed_groups_and_body(self):
+        """Test files with both grouped and non-grouped units."""
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="file1.txt" source-language="en" datatype="plaintext">
+        <body>
+            <group id="group1">
+                <trans-unit id="hello">
+                    <source>Hello</source>
+                </trans-unit>
+            </group>
+            <trans-unit id="world">
+                <source>World</source>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        translation = xliff.xlifffile()
+        translation.sourcelanguage = "en"
+        translation.targetlanguage = "fr"
+
+        for unit in source.units:
+            translation.addunit(unit)
+
+        # Verify structure is preserved
+        serialized = bytes(translation)
+        translation_reloaded = xliff.xlifffile.parsestring(serialized)
+        assert len(translation_reloaded.units) == 2
+
+        # First unit should be in group
+        parent1 = translation_reloaded.units[0].xmlelement.getparent()
+        assert etree.QName(parent1).localname == "group"
+
+        # Second unit should be directly in body
+        parent2 = translation_reloaded.units[1].xmlelement.getparent()
+        assert etree.QName(parent2).localname == "body"
+
+    def test_addunit_with_new_false(self):
+        """Test that addunit with new=False doesn't duplicate units."""
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="file1.txt" source-language="en" datatype="plaintext">
+        <body>
+            <group id="group1">
+                <trans-unit id="hello">
+                    <source>Hello</source>
+                </trans-unit>
+            </group>
+        </body>
+    </file>
+</xliff>"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        translation = xliff.xlifffile()
+        translation.sourcelanguage = "en"
+        translation.targetlanguage = "fr"
+
+        unit = source.units[0]
+
+        # Add unit with new=False - should not add to XML tree
+        translation.addunit(unit, new=False)
+
+        # Unit should be in units list but namespace should be set
+        assert len(translation.units) == 1
+        assert unit.namespace == translation.namespace
+
+        # But xmlelement should not be in the body or any group
+        serialized = bytes(translation)
+        # Should not contain the trans-unit since new=False
+        translation_reloaded = xliff.xlifffile.parsestring(serialized)
+        assert len(translation_reloaded.units) == 0
+
+    def test_namespace_preservation_across_versions(self):
+        """Test that namespace is properly handled when copying between XLIFF versions."""
+        # XLIFF 1.1 source
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="file1.txt" source-language="en" datatype="plaintext">
+        <body>
+            <group id="group1">
+                <trans-unit id="hello">
+                    <source>Hello</source>
+                </trans-unit>
+            </group>
+        </body>
+    </file>
+</xliff>"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        translation = xliff.xlifffile()
+        translation.sourcelanguage = "en"
+        translation.targetlanguage = "fr"
+
+        # Add unit and verify namespace is set correctly
+        unit = source.units[0]
+        translation.addunit(unit)
+
+        # Namespace should be updated to target's namespace
+        assert unit.namespace == translation.namespace
+
+        # Verify it serializes correctly
+        serialized = bytes(translation)
+        assert b'xmlns="urn:oasis:names:tc:xliff:document:1.1"' in serialized
+
+    def test_add_units_between_different_files(self):
+        """Test adding units from one file with multiple <file> tags to another, preserving file structure."""
+        # Source with two files
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="source_file1.txt" source-language="en" datatype="plaintext">
+        <body>
+            <trans-unit id="hello">
+                <source>Hello</source>
+            </trans-unit>
+        </body>
+    </file>
+    <file original="source_file2.txt" source-language="en" datatype="plaintext">
+        <body>
+            <trans-unit id="world">
+                <source>World</source>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+
+        # Target with one file
+        target_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="target_file.txt" source-language="en" datatype="plaintext" target-language="fr">
+        <body>
+            <trans-unit id="existing">
+                <source>Existing</source>
+                <target>Existent</target>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        target = xliff.xlifffile.parsestring(target_xliff)
+
+        # Add units from source to target
+        for unit in source.units:
+            target.addunit(unit)
+
+        # Verify all three files are present
+        serialized = bytes(target)
+        target_reloaded = xliff.xlifffile.parsestring(serialized)
+
+        filenames = target_reloaded.getfilenames()
+        assert len(filenames) == 3
+        assert "target_file.txt" in filenames
+        assert "source_file1.txt" in filenames
+        assert "source_file2.txt" in filenames
+
+        # Verify units are in correct files
+        assert len(target_reloaded.units) == 3
+        for unit in target_reloaded.units:
+            unit_id = unit.getid()
+            if "existing" in unit_id:
+                assert unit_id.startswith("target_file.txt")
+            elif "hello" in unit_id:
+                assert unit_id.startswith("source_file1.txt")
+            elif "world" in unit_id:
+                assert unit_id.startswith("source_file2.txt")
+
+    def test_group_preservation_across_different_namespaces(self):
+        """Test that group structure is preserved when adding units, using namespace-independent matching."""
+        # Source with groups
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="source.txt" source-language="en" datatype="plaintext">
+        <body>
+            <group id="mygroup" restype="x-gettext-domain">
+                <trans-unit id="hello">
+                    <source>Hello</source>
+                </trans-unit>
+            </group>
+        </body>
+    </file>
+</xliff>"""
+
+        # Target without groups (but same namespace)
+        target_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="target.txt" source-language="en" datatype="plaintext" target-language="fr">
+        <body>
+            <trans-unit id="existing">
+                <source>Existing</source>
+                <target>Existent</target>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        target = xliff.xlifffile.parsestring(target_xliff)
+
+        # Verify initial state
+        assert len(source.units) == 1
+        assert len(target.units) == 1
+
+        # Add unit with group from source to target
+        target.addunit(source.units[0])
+
+        # Verify target now has 2 units
+        assert len(target.units) == 2
+
+        # Serialize and reload to verify structure
+        serialized = bytes(target)
+        target_reloaded = xliff.xlifffile.parsestring(serialized)
+
+        # Should still have 2 units after round-trip
+        assert len(target_reloaded.units) == 2
+
+        # Find the newly added unit by source text
+        new_unit = None
+        existing_unit = None
+        for unit in target_reloaded.units:
+            if unit.source == "Hello":
+                new_unit = unit
+            elif unit.source == "Existing":
+                existing_unit = unit
+
+        assert new_unit is not None, "New unit with 'Hello' not found"
+        assert existing_unit is not None, "Existing unit not found"
+
+        # Verify the new unit is in a group (namespace-independent check)
+        new_parent = new_unit.xmlelement.getparent()
+        assert etree.QName(new_parent).localname == "group", (
+            "New unit should be in a group"
+        )
+        assert new_parent.get("id") == "mygroup"
+        assert new_parent.get("restype") == "x-gettext-domain"
+
+        # Verify the existing unit is NOT in a group
+        existing_parent = existing_unit.xmlelement.getparent()
+        assert etree.QName(existing_parent).localname == "body", (
+            "Existing unit should be in body"
+        )
+
+        # Verify unit's namespace is correctly set to target's namespace
+        assert new_unit.namespace == target.namespace
+
+    def test_cross_namespace_group_and_file_preservation(self):
+        """Test that groups and files are preserved when adding units across different XLIFF namespaces."""
+        # Source with XLIFF 1.2 namespace, with groups
+        source_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.2" xmlns="urn:oasis:names:tc:xliff:document:1.2">
+    <file original="source_file1.txt" source-language="en" datatype="plaintext">
+        <body>
+            <group id="group1" restype="x-gettext-domain">
+                <trans-unit id="hello">
+                    <source>Hello</source>
+                </trans-unit>
+            </group>
+        </body>
+    </file>
+    <file original="source_file2.txt" source-language="en" datatype="plaintext">
+        <body>
+            <trans-unit id="world">
+                <source>World</source>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+
+        # Target with XLIFF 1.1 namespace (different from source)
+        target_xliff = b"""<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+    <file original="target_file.txt" source-language="en" datatype="plaintext" target-language="fr">
+        <body>
+            <trans-unit id="existing">
+                <source>Existing</source>
+                <target>Existent</target>
+            </trans-unit>
+        </body>
+    </file>
+</xliff>"""
+
+        source = xliff.xlifffile.parsestring(source_xliff)
+        target = xliff.xlifffile.parsestring(target_xliff)
+
+        # Verify different namespaces
+        assert source.namespace == "urn:oasis:names:tc:xliff:document:1.2"
+        assert target.namespace == "urn:oasis:names:tc:xliff:document:1.1"
+
+        # Add units from source (1.2) to target (1.1)
+        for unit in source.units:
+            target.addunit(unit)
+
+        # Check that units were registered
+        assert len(target.units) == 3, f"Expected 3 units, got {len(target.units)}"
+
+        # Verify files are preserved - target should have original file plus two from source
+        filenames = target.getfilenames()
+        assert len(filenames) == 3, (
+            f"Expected 3 files, got {len(filenames)}: {filenames}"
+        )
+        assert "target_file.txt" in filenames
+        assert "source_file1.txt" in filenames
+        assert "source_file2.txt" in filenames
+
+        # Serialize and check structure
+        serialized = bytes(target)
+
+        # Parse with lxml to check namespace and structure
+
+        tree = etree.fromstring(serialized)
+
+        # Verify the document uses target's namespace (1.1)
+        assert tree.nsmap[None] == "urn:oasis:names:tc:xliff:document:1.1"
+
+        # Check for groups in target namespace
+        groups = tree.xpath(
+            "//ns:group", namespaces={"ns": "urn:oasis:names:tc:xliff:document:1.1"}
+        )
+        assert len(groups) >= 1, f"Expected at least 1 group, found {len(groups)}"
+
+        # Verify group attributes are preserved
+        group1 = None
+        for g in groups:
+            if g.get("id") == "group1":
+                group1 = g
+                break
+        assert group1 is not None, "group1 not found"
+        assert group1.get("restype") == "x-gettext-domain"
+
+        # Verify all files are present in target namespace
+        files = tree.xpath(
+            "//ns:file", namespaces={"ns": "urn:oasis:names:tc:xliff:document:1.1"}
+        )
+        assert len(files) == 3, f"Expected 3 file elements, found {len(files)}"
+
+        file_originals = [f.get("original") for f in files]
+        assert "target_file.txt" in file_originals
+        assert "source_file1.txt" in file_originals
+        assert "source_file2.txt" in file_originals
+
     def test_indent(self):
         xlfsource = b"""<?xml version="1.0" encoding="UTF-8"?>
 <xliff xmlns="urn:oasis:names:tc:xliff:document:1.1" version="1.1">
