@@ -24,8 +24,11 @@ for examples and usage instructions.
 """
 
 import logging
+import re
 
+from translate.convert import convert
 from translate.convert.accesskey import UnitMixer
+from translate.lang import data
 from translate.storage import po, properties
 
 logger = logging.getLogger(__name__)
@@ -194,10 +197,6 @@ class prop2po:
                 self.unit = unit
                 self.variants = {}
 
-        import re
-
-        from translate.lang import data
-
         regex = re.compile(r"([^\[\]]*)(?:\[(.*)\])?")
         names = data.cldr_plural_categories
         new_store = type(postore)()
@@ -311,7 +310,6 @@ class prop2po:
         if current_plural:
             # The file ended with a set of plural units
             _append_plural_unit(new_store, plurals, current_plural)
-            current_plural = ""
 
         # if everything went well, there should be nothing left in plurals
         if len(plurals) != 0:
@@ -320,8 +318,7 @@ class prop2po:
             )
         return new_store
 
-    @staticmethod
-    def convertunit(propunit, commenttype):
+    def convertunit(self, propunit, commenttype):
         """
         Converts a .properties unit to a .po unit. Returns None if empty or
         not for translation.
@@ -339,8 +336,21 @@ class prop2po:
         if propunit.isblank():
             return None
         pounit.addlocation(propunit.name)
-        pounit.source = propunit.source
-        pounit.target = ""
+
+        # For .strings files, treat them as bilingual:
+        # - The key (name) is the source text (msgid)
+        # - The value (source property) is the translated text (msgstr)
+        if self.personality in {"strings", "strings-utf8"}:
+            # Decode the key to get the actual source text
+            personality_obj = properties.get_dialect(self.personality)
+            pounit.source = personality_obj.decode(
+                personality_obj.key_strip(propunit.name)
+            )
+            pounit.target = propunit.source
+        else:
+            # For other formats, use standard monolingual behavior
+            pounit.source = propunit.source
+            pounit.target = ""
         return pounit
 
     def convertmixedunit(self, labelprop, accesskeyprop, commenttype):
@@ -475,8 +485,6 @@ formats = {
 
 
 def main(argv=None):
-    from translate.convert import convert
-
     parser = convert.ConvertOptionParser(
         formats, usetemplates=True, usepots=True, description=__doc__
     )
