@@ -160,13 +160,13 @@ class StringElem:
 
     def __repr__(self):
         elemstr = ", ".join(repr(elem) for elem in self.sub)
-        return "<%(class)s(%(id)s%(rid)s%(xid)s[%(subs)s])>" % {
-            "class": self.__class__.__name__,
-            "id": (self.id is not None and f'id="{self.id}" ') or "",
-            "rid": (self.rid is not None and f'rid="{self.rid}" ') or "",
-            "xid": (self.xid is not None and f'xid="{self.xid}" ') or "",
-            "subs": elemstr,
-        }
+        return "<{class_name}({id}{rid}{xid}[{subs}])>".format(
+            class_name=self.__class__.__name__,
+            id=(self.id is not None and f'id="{self.id}" ') or "",
+            rid=(self.rid is not None and f'rid="{self.rid}" ') or "",
+            xid=(self.xid is not None and f'xid="{self.xid}" ') or "",
+            subs=elemstr,
+        )
 
     def __str__(self):
         if callable(self.renderer):
@@ -184,9 +184,9 @@ class StringElem:
                   string or unicode.
         """
         for elem in self.flatten():
-            for i in range(len(elem.sub)):
-                if isinstance(elem.sub[i], str):
-                    elem.sub[i] = f(elem.sub[i])
+            for i, subelement in enumerate(elem.sub):
+                if isinstance(subelement, str):
+                    elem.sub[i] = f(subelement)
 
     def copy(self):
         """
@@ -212,8 +212,8 @@ class StringElem:
         if parent is None:
             raise ElementNotFoundError(repr(elem))
         subidx = -1
-        for i in range(len(parent.sub)):
-            if parent.sub[i] is elem:
+        for i, subelement in enumerate(parent.sub):
+            if subelement is elem:
                 subidx = i
                 break
         if subidx < 0:
@@ -237,13 +237,11 @@ class StringElem:
         if start_index == end_index:
             return StringElem(), self, 0
         if start_index > end_index:
-            raise IndexError(
-                "start_index > end_index: %d > %d" % (start_index, end_index)
-            )
+            raise IndexError(f"start_index > end_index: {start_index} > {end_index}")
         if start_index < 0 or start_index > len(self):
-            raise IndexError("start_index: %d" % (start_index))
+            raise IndexError(f"start_index: {start_index}")
         if end_index < 1 or end_index > len(self) + 1:
-            raise IndexError("end_index: %d" % (end_index))
+            raise IndexError(f"end_index: {end_index}")
 
         start = self.get_index_data(start_index)
         if isinstance(start["elem"], tuple):
@@ -326,9 +324,6 @@ class StringElem:
             newstr = "".join(start["elem"].sub)
             removed = StringElem(newstr[start["offset"] : end["offset"]])
             newstr = newstr[: start["offset"]] + newstr[end["offset"] :]
-            parent = self.get_parent_elem(start["elem"])
-            if parent is None and start["elem"] is self:
-                parent = self
             start["elem"].sub = [newstr]
             self.prune()
             return removed, start["elem"], start["offset"]
@@ -337,10 +332,10 @@ class StringElem:
         range_nodes = self.depth_first()
         startidx = 0
         endidx = -1
-        for i in range(len(range_nodes)):
-            if range_nodes[i] is start["elem"]:
+        for i, range_node in enumerate(range_nodes):
+            if range_node is start["elem"]:
                 startidx = i
-            elif range_nodes[i] is end["elem"]:
+            elif range_node is end["elem"]:
                 endidx = i
                 break
         range_nodes = range_nodes[startidx : endidx + 1]
@@ -540,7 +535,7 @@ class StringElem:
         string (Unicode) representation.
         """
         if offset < 0 or offset > len(self):
-            raise IndexError("Index out of range: %d" % (offset))
+            raise IndexError(f"Index out of range: {offset}")
         if isinstance(text, str):
             text = StringElem(text)
         if not isinstance(text, StringElem):
@@ -854,7 +849,7 @@ class StringElem:
         indent_prefix = " " * indent * 2
         out = f"{indent_prefix}{self.__class__.__name__} [{self!s}]"
         if verbose:
-            out += " " + repr(self)
+            out += f" {self!r}"
 
         print(out)
 
