@@ -28,12 +28,10 @@ from __future__ import annotations
 import copy
 import logging
 import re
-import textwrap
-from functools import lru_cache
 from itertools import chain
-from warnings import warn
+from string import punctuation
 
-from wcwidth import wcswidth
+from unicode_segmentation_rs import gettext_wrap
 
 from translate.misc import quote
 from translate.misc.multistring import multistring
@@ -58,12 +56,17 @@ po_unescape_re = re.compile(r"\\.")
 po_escape_map = {value: key for (key, value) in po_unescape_map.items()}
 po_escape_re = re.compile("|".join(re.escape(key) for key in po_escape_map))
 
+po_line_break_chars = {"/", "}", ")", ">", "-"}
+po_mergeable_chars = po_line_break_chars | {" ", "\t"}
+po_open_parenthesis_chars = {"{", "("}
+po_punctuation = set(punctuation)
+
 
 def splitlines(text):
     """
     Split lines based on first newline char.
 
-    Can not use univerzal newlines as they match any newline like
+    Can not use universal newlines as they match any newline like
     character inside text and that breaks on files with unix newlines
     and LF chars inside comments.
 
@@ -102,138 +105,35 @@ def escapeforpo(line: str) -> str:
     return po_escape_re.sub(escapehandler, line)
 
 
-@lru_cache(maxsize=128)
-def unicode_width(text: str) -> int:
-    result = wcswidth(text)
-    if result == -1:
-        warn(
-            f"String contains control characters or non-displayable characters: {text!r}"
-        )
-        return len(text)
-    return result
+class PoWrapper:
+    r"""
+    Gettext-compatible text wrapper for PO files.
 
-
-def cjkslices(text: str, index: int) -> tuple[str, str]:
-    """Return the two slices of a text cut to the index."""
-    text_width = unicode_width(text)
-    # Do not wrap non-wide chars here
-    if text_width == len(text):
-        return text, ""
-
-    # Guess split point
-    split_point = index * text_width // len(text)
-    while unicode_width(text[:split_point]) < index:
-        split_point += 1
-
-    while unicode_width(text[:split_point]) > index:
-        split_point -= 1
-
-    return text[:split_point], text[split_point:]
-
-
-class PoWrapper(textwrap.TextWrapper):
+    This implementation follows gettext's wrapping behavior:
+    - Never breaks escape sequences (\\n, \\", etc.)
+    - Prefers breaking after spaces
+    - Handles CJK characters with proper width calculation
+    - Breaks long words only when necessary
     """
-    Customized TextWrapper.
-
-    - custom word separator regexp
-    - full width chars accounting, based on https://bugs.python.org/issue24665
-    - dropped support for unused features (for example max_lines or drop_whitespace)
-    """
-
-    wordsep_re = re.compile(
-        r"""
-            (
-            \[[^\]]{1,73}\]|                      # [] braces
-            \([^\)]{1,73}\)|                      # () braces
-            \\"[^"]{1,73}\\"|                     # quoted string
-            \s+|                                  # any whitespace
-            [a-z0-9A-Z_#\[\].-]+/|                # nicely split long URLs
-            \w*\\.\w*|                            # any escape should not be split
-            [\w\!\'\&\.\,\?=<>%]+\s+|             # space should go with a word
-            [，。、]|                             # full width punctuation
-            [^\s\w]*\w+[a-zA-Z]-(?=\w+[a-zA-Z])|  # hyphenated words
-            (?<=[\w\!\"\'\&\.\,\?])-{2,}(?=\w)    # em-dash
-            )
-        """,
-        re.VERBOSE,
-    )
 
     def __init__(self, width: int = 77) -> None:
-        super().__init__(
-            width=width,
-            replace_whitespace=False,
-            expand_tabs=False,
-            drop_whitespace=False,
-            break_long_words=True,
-        )
+        self.width = width
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: object) -> bool:
         if not isinstance(other, PoWrapper):
             return False
         return self.width == other.width
 
-    def _handle_long_word(
-        self, reversed_chunks: list[str], cur_line: list[str], cur_len: int, width: int
-    ) -> None:
+    def wrap(self, text: str) -> list[str]:
         """
-        Handle a chunk of text (most likely a word, not whitespace) that
-        is too long to fit in any line.
+        Wrap text to fit within the specified width.
+
+        Returns a list of lines, each no longer than self.width characters
+        (accounting for character display width for CJK characters).
         """
-        # We're allowed to break long words, then do so: put as much
-        # of the next chunk onto the current line as will fit.
-        chunk_start, chunk_end = cjkslices(reversed_chunks[-1], width - cur_len)
-        cur_line.append(chunk_start)
-        reversed_chunks[-1] = chunk_end
-
-    def _wrap_chunks(self, chunks: list[str]) -> list[str]:
-        lines = []
-        width = self.width
-        if width <= 1:
-            raise ValueError(f"invalid width {self.width!r} (must be > 1)")
-        if self.subsequent_indent or self.initial_indent:
-            raise ValueError("PoWrapper does not support indent")
-
-        # Arrange in reverse order so items can be efficiently popped
-        # from a stack of chucks.
-        chunks.reverse()
-
-        while chunks:
-            # Start the list of chunks that will make up the current line.
-            # cur_len is just the length of all the chunks in cur_line.
-            cur_line = []
-            cur_len = 0
-
-            while chunks:
-                l = unicode_width(chunks[-1])
-
-                # Can at least squeeze this chunk onto the current line.
-                if cur_len + l <= width:
-                    cur_line.append(chunks.pop())
-                    cur_len += l
-
-                # Figure out if we can split wide chars
-                elif l > len(chunks[-1]):
-                    slice1, slice2 = cjkslices(chunks[-1], width - cur_len)
-                    if slice2:
-                        cur_line.append(slice1)
-                        cur_len += unicode_width(slice1)
-                        chunks[-1] = slice2
-                    break
-
-                # Nope, this line is full.
-                else:
-                    break
-
-            # The current line is full, and the next chunk is too big to
-            # fit on *any* line (not just this one).
-            if not cur_line and chunks and unicode_width(chunks[-1]) > width:
-                self._handle_long_word(chunks, cur_line, cur_len, width)
-
-            if cur_line:
-                # Convert current line back to a string and store it in
-                # list of all lines (return value).
-                lines.append("".join(cur_line))
-        return lines
+        if not text or self.width <= 0:
+            return [text] if text else []
+        return gettext_wrap(text, self.width)
 
 
 def quoteforpo(text: str | None, wrapper_obj: PoWrapper | None = None) -> list[str]:
@@ -245,21 +145,13 @@ def quoteforpo(text: str | None, wrapper_obj: PoWrapper | None = None) -> list[s
     text = escapeforpo(text)
     if wrapper_obj.width == -1:
         return [f'"{text}"']
-    lines = text.split("\\n")
-    for i, l in enumerate(lines[:-1]):
-        lines[i] = l + "\\n"
+
+    lines = wrapper_obj.wrap(text)
 
     polines = []
-    len_lines = len(lines)
-    if (
-        len_lines > 2
-        or (len_lines == 2 and lines[1])
-        or len(lines[0]) > wrapper_obj.width - 6
-    ):
+    if len(lines) >= 2 or (lines and len(lines[0]) > wrapper_obj.width - 6):
         polines.append('""')
-    for line in lines:
-        lns = wrapper_obj.wrap(line)
-        polines.extend(f'"{ln}"' for ln in lns)
+    polines.extend(f'"{line}"' for line in lines)
     return polines
 
 
@@ -280,7 +172,7 @@ def unescape(line: str) -> str:
     return po_unescape_re.sub(unescapehandler, line)
 
 
-def unquotefrompo(postr: str) -> str:
+def unquotefrompo(postr: list[str]) -> str:
     return "".join(unescape(line[1:-1]) for line in postr)
 
 
@@ -314,18 +206,18 @@ class pounit(pocommon.pounit):
     # fashion
     __shallow__ = ["_store", "wrapper"]
 
-    def __init__(self, source=None, wrapper=None, **kwargs):
-        self.wrapper = wrapper
-        self.obsolete = False
+    def __init__(self, source=None, wrapper: PoWrapper | None = None, **kwargs):
+        self.wrapper: PoWrapper | None = wrapper
+        self.obsolete: bool = False
         self._initallcomments(blankall=True)
-        self.prev_msgctxt = []
-        self.prev_msgid = []
-        self.prev_msgid_plural = []
-        self.msgctxt = []
-        self.msgid = []
-        self.msgid_pluralcomments = []
-        self.msgid_plural = []
-        self.msgstr = []
+        self.prev_msgctxt: list[str] = []
+        self.prev_msgid: list[str] = []
+        self.prev_msgid_plural: list[str] = []
+        self.msgctxt: list[str] = []
+        self.msgid: list[str] = []
+        self.msgid_pluralcomments: list[str] = []
+        self.msgid_plural: list[str] = []
+        self.msgstr: list[str] | dict[int, list[str]] = []
         super().__init__(source)
 
     @property
@@ -361,21 +253,19 @@ class pounit(pocommon.pounit):
             return multistring([singular, pluralform])
         return singular
 
-    def quote(self, text):
+    def quote(self, text: str) -> list[str]:
         return quoteforpo(text, self.wrapper)
 
-    def _set_source_vars(self, source):
-        msgid = None
-        msgid_plural = None
+    def _set_source_vars(
+        self, source: str | list[str] | multistring
+    ) -> tuple[list[str], list[str]]:
         if isinstance(source, multistring):
             source = source.strings
         if isinstance(source, list):
-            msgid = self.quote(source[0])
-            msgid_plural = self.quote(source[1]) if len(source) > 1 else []
-        else:
-            msgid = self.quote(source)
-            msgid_plural = []
-        return msgid, msgid_plural
+            return self.quote(source[0]), self.quote(source[1]) if len(
+                source
+            ) > 1 else []
+        return self.quote(source), []
 
     @property
     def source(self):
@@ -383,7 +273,7 @@ class pounit(pocommon.pounit):
         return self._get_source_vars(self.msgid, self.msgid_plural)
 
     @source.setter
-    def source(self, source):
+    def source(self, source: str | list[str] | multistring) -> None:
         """
         Sets the msgid to the given (unescaped) value.
 
@@ -427,8 +317,7 @@ class pounit(pocommon.pounit):
                 target = target[0]
             else:
                 raise ValueError(
-                    "po msgid element has no plural but msgstr has %d elements (%s)"
-                    % (len(target), target)
+                    f"po msgid element has no plural but msgstr has {len(target)} elements ({target})"
                 )
         templates = self.msgstr
         if isinstance(templates, list):
@@ -453,10 +342,6 @@ class pounit(pocommon.pounit):
         if prev_source and self.isfuzzy():
             unit = type(self)(prev_source)
             unit.target = self.target
-            # Already released versions of Virtaal (0.6.x) only supported XLIFF
-            # alternatives, and expect .xmlelement.get().
-            # This can be removed soon:
-            unit.xmlelement = {}
             return [unit]
         return []
 
@@ -546,9 +431,9 @@ class pounit(pocommon.pounit):
             return sum(len(unquotefrompo(msgstr)) for msgstr in self.msgstr.values())
         return len(unquotefrompo(self.msgstr))
 
-    def merge(self, otherpo, overwrite=False, comments=True, authoritative=False):
+    def merge(self, otherunit, overwrite=False, comments=True, authoritative=False):
         """
-        Merges the otherpo (with the same msgid) into this one.
+        Merges the otherunit (with the same msgid) into this one.
 
         Overwrite non-blank self.msgstr only if overwrite is True
         merge comments only if comments is True
@@ -593,37 +478,37 @@ class pounit(pocommon.pounit):
                     if item not in list1 or len(item) < 5:
                         list1.append(item)
 
-        if not isinstance(otherpo, pounit):
-            super().merge(otherpo, overwrite, comments)
+        if not isinstance(otherunit, pounit):
+            super().merge(otherunit, overwrite, comments)
             return
         if comments:
-            mergelists(self.othercomments, otherpo.othercomments)
-            mergelists(self.typecomments, otherpo.typecomments)
+            mergelists(self.othercomments, otherunit.othercomments)
+            mergelists(self.typecomments, otherunit.typecomments)
             if not authoritative:
-                # We don't bring across otherpo.automaticcomments as we
+                # We don't bring across otherunit.automaticcomments as we
                 # consider ourself to be the the authority.  Same applies
-                # to otherpo.msgidcomments
-                mergelists(self.automaticcomments, otherpo.automaticcomments)
-                mergelists(self.msgidcomments, otherpo.msgidcomments)
-                mergelists(self.sourcecomments, otherpo.sourcecomments, split=True)
+                # to otherunit.msgidcomments
+                mergelists(self.automaticcomments, otherunit.automaticcomments)
+                mergelists(self.msgidcomments, otherunit.msgidcomments)
+                mergelists(self.sourcecomments, otherunit.sourcecomments, split=True)
         if not self.istranslated() or overwrite:
             # Remove kde-style comments from the translation (if any).
-            if self._extract_msgidcomments(otherpo.target):
-                otherpo.target = otherpo.target.replace(
-                    "_: " + otherpo._extract_msgidcomments() + self.newline, ""
+            if self._extract_msgidcomments(otherunit.target):
+                otherunit.target = otherunit.target.replace(
+                    f"_: {otherunit._extract_msgidcomments()}{self.newline}", ""
                 )
-            self.target = otherpo.target
+            self.target = otherunit.target
             if (
-                self.source != otherpo.source
-                or self.getcontext() != otherpo.getcontext()
+                self.source != otherunit.source
+                or self.getcontext() != otherunit.getcontext()
             ):
                 self.markfuzzy()
             else:
-                self.markfuzzy(otherpo.isfuzzy())
-        elif not otherpo.istranslated():
-            if self.source != otherpo.source:
+                self.markfuzzy(otherunit.isfuzzy())
+        elif not otherunit.istranslated():
+            if self.source != otherunit.source:
                 self.markfuzzy()
-        elif self.target != otherpo.target:
+        elif self.target != otherunit.target:
             self.markfuzzy()
 
     def isheader(self):
@@ -738,7 +623,7 @@ class pounit(pocommon.pounit):
             partkeys = sorted(partlines.keys())
             return "".join(
                 self._getmsgpartstr(
-                    "%s[%d]" % (partname, partkey), partlines[partkey], partcomments
+                    f"{partname}[{partkey}]", partlines[partkey], partcomments
                 )
                 for partkey in partkeys
             )
